@@ -28,7 +28,7 @@
  * SoupBodyInputStreamHttp2
  * @short_description: Streaming input operations on memory chunks
  *
- * [type@BodyInputStreamHttp2] is a class for using arbitrary
+ * #SoupBodyInputStreamHttp2 is a class for using arbitrary
  * memory chunks as input for GIO streaming input operations.
  *
  * It differs from #GMemoryInputStream in that it frees older chunks
@@ -42,7 +42,7 @@ struct _SoupBodyInputStreamHttp2 {
 };
 
 typedef struct {
-        GQueue *chunks;
+        GSList *chunks;
         gsize start_offset;
         gsize len;
         gsize pos;
@@ -67,7 +67,7 @@ static guint signals [LAST_SIGNAL] = { 0 };
 /**
  * soup_body_input_stream_http2_new:
  *
- * Creates a new empty [type@BodyInputStreamHttp2].
+ * Creates a new empty #SoupBodyInputStreamHttp2. 
  *
  * Returns: a new #GInputStream
  */
@@ -89,7 +89,7 @@ soup_body_input_stream_http2_add_data (SoupBodyInputStreamHttp2 *stream,
 
         priv = soup_body_input_stream_http2_get_instance_private (stream);
 
-        g_queue_push_tail (priv->chunks, g_bytes_new (data, size));
+        priv->chunks = g_slist_append (priv->chunks, g_bytes_new (data, size));
         priv->len += size;
         if (priv->need_more_data_cancellable) {
                 g_cancellable_cancel (priv->need_more_data_cancellable);
@@ -118,7 +118,7 @@ soup_body_input_stream_http2_read_real (GInputStream  *stream,
 {
         SoupBodyInputStreamHttp2 *memory_stream;
         SoupBodyInputStreamHttp2Private *priv;
-        GList *l;
+        GSList *l;
         GBytes *chunk;
         gsize len;
         gsize offset, start, rest, size;
@@ -135,7 +135,7 @@ soup_body_input_stream_http2_read_real (GInputStream  *stream,
         count = MIN (read_count, priv->len - priv->pos);
 
         offset = priv->start_offset;
-        for (l = g_queue_peek_head_link(priv->chunks); l; l = l->next) {
+        for (l = priv->chunks; l; l = l->next) {
                 chunk = (GBytes *)l->data;
                 len = g_bytes_get_size (chunk);
 
@@ -150,7 +150,7 @@ soup_body_input_stream_http2_read_real (GInputStream  *stream,
         rest = count;
 
         while (l && rest > 0) {
-                GList *next = l->next;
+                GSList *next = l->next;
 
                 const guint8 *chunk_data;
                 chunk = (GBytes *)l->data;
@@ -165,7 +165,7 @@ soup_body_input_stream_http2_read_real (GInputStream  *stream,
                 /* Remove fully read chunk from list, note that we are always near the start of the list */
                 if (start + size == len) {
                         priv->start_offset += len;
-                        g_queue_delete_link (priv->chunks, l);
+                        priv->chunks = g_slist_delete_link (priv->chunks, l);
                         g_bytes_unref (chunk);
                 }
 
@@ -256,12 +256,12 @@ soup_body_input_stream_http2_skip (GInputStream  *stream,
 
         /* Remove all skipped chunks */
         gsize offset = priv->start_offset;
-        for (GList *l = g_queue_peek_head_link(priv->chunks); l; l = l->next) {
+        for (GSList *l = priv->chunks; l; l = l->next) {
                 GBytes *chunk = (GBytes *)l->data;
                 gsize chunk_len = g_bytes_get_size (chunk);
 
                 if (offset + chunk_len <= priv->pos) {
-                        g_queue_delete_link (priv->chunks, l);
+                        priv->chunks = g_slist_delete_link (priv->chunks, l);
                         g_bytes_unref (chunk);
                         offset += chunk_len;
                 }
@@ -351,13 +351,9 @@ soup_body_input_stream_http2_create_source (GPollableInputStream *stream,
         SoupBodyInputStreamHttp2Private *priv = soup_body_input_stream_http2_get_instance_private (SOUP_BODY_INPUT_STREAM_HTTP2 (stream));
         GSource *base_source, *pollable_source;
 
-        if (priv->pos < priv->len) {
-                base_source = g_timeout_source_new (0);
-        } else {
-                if (!priv->need_more_data_cancellable)
-                        priv->need_more_data_cancellable = g_cancellable_new ();
-                base_source = g_cancellable_source_new (priv->need_more_data_cancellable);
-        }
+        if (!priv->need_more_data_cancellable)
+                priv->need_more_data_cancellable = g_cancellable_new ();
+        base_source = g_cancellable_source_new (priv->need_more_data_cancellable);
 
         pollable_source = g_pollable_source_new_full (stream, base_source, cancellable);
         g_source_set_name (pollable_source, "SoupMemoryStreamSource");
@@ -387,7 +383,7 @@ soup_body_input_stream_http2_finalize (GObject *object)
         SoupBodyInputStreamHttp2 *stream = SOUP_BODY_INPUT_STREAM_HTTP2 (object);
         SoupBodyInputStreamHttp2Private *priv = soup_body_input_stream_http2_get_instance_private (stream);
 
-        g_queue_free_full (priv->chunks, (GDestroyNotify)g_bytes_unref);
+        g_slist_free_full (priv->chunks, (GDestroyNotify)g_bytes_unref);
 
         G_OBJECT_CLASS (soup_body_input_stream_http2_parent_class)->finalize (object);
 }
@@ -403,10 +399,6 @@ soup_body_input_stream_http2_pollable_iface_init (GPollableInputStreamInterface
 static void
 soup_body_input_stream_http2_init (SoupBodyInputStreamHttp2 *stream)
 {
-        SoupBodyInputStreamHttp2Private *priv;
-
-        priv = soup_body_input_stream_http2_get_instance_private (stream);
-        priv->chunks = g_queue_new ();
 }
 
 static void
