@@ -170,7 +170,11 @@ soup_connection_set_property (GObject *object, guint prop_id,
         case PROP_CONTEXT:
                 priv->idle_timeout_src = g_timeout_source_new (0);
                 g_source_set_ready_time (priv->idle_timeout_src, -1);
+#if GLIB_CHECK_VERSION(2, 70, 0)
+                g_source_set_static_name (priv->idle_timeout_src, "Soup connection idle timeout");
+#else
                 g_source_set_name (priv->idle_timeout_src, "Soup connection idle timeout");
+#endif
                 g_source_set_callback (priv->idle_timeout_src, idle_timeout, object, NULL);
                 g_source_attach (priv->idle_timeout_src, g_value_get_pointer (value));
                 break;
@@ -794,6 +798,7 @@ soup_connection_connect_async (SoupConnection      *conn,
 
         priv->cancellable = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
         task = g_task_new (conn, priv->cancellable, callback, user_data);
+        g_task_set_source_tag (task, soup_connection_connect_async);
         g_task_set_priority (task, io_priority);
 
         client = new_socket_client (conn);
@@ -919,6 +924,7 @@ soup_connection_tunnel_handshake_async (SoupConnection     *conn,
 
         priv->cancellable = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
         task = g_task_new (conn, priv->cancellable, callback, user_data);
+        g_task_set_source_tag (task, soup_connection_tunnel_handshake_async);
         g_task_set_priority (task, io_priority);
 
         tls_connection = new_tls_connection (conn, G_SOCKET_CONNECTION (priv->connection), &error);
