@@ -360,7 +360,7 @@ do_post_large_sync_test (Test *test, gconstpointer data)
         GUri *uri;
         SoupMessage *msg;
         GInputStream *response;
-        guint large_size = 1000000;
+        guint large_size = 10000;
         char *large_data;
         unsigned int i;
         GError *error = NULL;
@@ -422,7 +422,7 @@ do_post_large_async_test (Test *test, gconstpointer data)
         SoupMessage *msg;
         GBytes *response = NULL;
         GMainContext *async_context = g_main_context_ref_thread_default ();
-        guint large_size = 1000000;
+        guint large_size = 10000;
         char *large_data;
         unsigned int i;
 
@@ -553,90 +553,6 @@ do_paused_async_test (Test *test, gconstpointer data)
         g_uri_unref (uri);
 }
 
-typedef struct {
-        int connection;
-        int stream;
-} WindowSize;
-
-static void
-flow_control_message_network_event (SoupMessage        *msg,
-                                    GSocketClientEvent  event,
-                                    GIOStream          *connection,
-                                    WindowSize         *window_size)
-{
-        SoupConnection *conn;
-
-        if (event != G_SOCKET_CLIENT_RESOLVING)
-                return;
-
-        conn = soup_message_get_connection (msg);
-        g_assert_nonnull (conn);
-        if (window_size->connection != -1)
-                soup_connection_set_http2_initial_window_size (conn, window_size->connection);
-        if (window_size->stream != -1)
-                soup_connection_set_http2_initial_stream_window_size (conn, window_size->stream);
-}
-
-static void
-do_flow_control_large_test (Test *test, gconstpointer data)
-{
-        gboolean async = GPOINTER_TO_INT (data);
-        GUri *uri;
-        SoupMessage *msg;
-        GBytes *response;
-        GError *error = NULL;
-        WindowSize window_size = { (LARGE_N_CHARS * LARGE_CHARS_REPEAT) / 2 , (LARGE_N_CHARS * LARGE_CHARS_REPEAT) / 2 };
-
-        uri = g_uri_parse_relative (base_uri, "/large", SOUP_HTTP_URI_FLAGS, NULL);
-        msg = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
-        g_signal_connect (msg, "network-event",
-                          G_CALLBACK (flow_control_message_network_event),
-                          &window_size);
-
-        if (async)
-                response = soup_test_session_async_send (test->session, msg, NULL, &error);
-        else
-                response = soup_session_send_and_read (test->session, msg, NULL, &error);
-
-        g_assert_no_error (error);
-        g_assert_cmpuint (g_bytes_get_size (response), ==, (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1);
-
-        g_uri_unref (uri);
-        g_bytes_unref (response);
-        g_object_unref (msg);
-}
-
-static void
-do_flow_control_multi_message_async_test (Test *test, gconstpointer data)
-{
-        GUri *uri;
-        SoupMessage *msg1, *msg2;
-        GBytes *response1 = NULL;
-        GBytes *response2 = NULL;
-        WindowSize window_size = { (LARGE_N_CHARS * LARGE_CHARS_REPEAT), (LARGE_N_CHARS * LARGE_CHARS_REPEAT) / 2 };
-
-        uri = g_uri_parse_relative (base_uri, "/large", SOUP_HTTP_URI_FLAGS, NULL);
-        msg1 = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
-        g_signal_connect (msg1, "network-event",
-                          G_CALLBACK (flow_control_message_network_event),
-                          &window_size);
-        msg2 = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
-        soup_session_send_async (test->session, msg1, G_PRIORITY_DEFAULT, NULL, on_send_complete, &response1);
-        soup_session_send_async (test->session, msg2, G_PRIORITY_DEFAULT, NULL, on_send_complete, &response2);
-
-        while (!response1 || !response2)
-                g_main_context_iteration (NULL, TRUE);
-
-        g_assert_cmpuint (g_bytes_get_size (response1), ==, (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1);
-        g_assert_cmpuint (g_bytes_get_size (response2), ==, (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1);
-
-        g_uri_unref (uri);
-        g_bytes_unref (response1);
-        g_bytes_unref (response2);
-        g_object_unref (msg1);
-        g_object_unref (msg2);
-}
-
 static SoupConnection *last_connection;
 
 static void
@@ -679,10 +595,10 @@ do_connections_test (Test *test, gconstpointer data)
         guint complete_count = 0;
         GUri *uri;
 
-#ifdef __SANITIZE_ADDRESS__
-        g_test_skip ("Flakey on asan GitLab runner");
-        return;
-#endif
+        if (g_getenv ("ASAN_OPTIONS")) {
+                g_test_skip ("Flakey on asan GitLab runner");
+                return;
+        }
 
         async_context = g_main_context_ref_thread_default ();
 
@@ -760,7 +676,6 @@ do_logging_test (Test *test, gconstpointer data)
         SoupLogger *logger = soup_logger_new (SOUP_LOGGER_LOG_BODY);
         soup_logger_set_printer (logger, log_printer, &has_logged_body, NULL);
         soup_session_add_feature (test->session, SOUP_SESSION_FEATURE (logger));
-        g_clear_object (&logger);
 
         uri = g_uri_parse_relative (base_uri, "/echo_post", SOUP_HTTP_URI_FLAGS, NULL);
         msg = soup_message_new_from_uri (SOUP_METHOD_POST, uri);
@@ -772,7 +687,6 @@ do_logging_test (Test *test, gconstpointer data)
         g_assert_true (has_logged_body);
 
         g_bytes_unref (response);
-        g_bytes_unref (bytes);
         g_object_unref (msg);
         g_uri_unref (uri);
 }
@@ -1080,7 +994,7 @@ do_invalid_header_rfc9113_received_test (Test *test, gconstpointer data)
 
         g_assert_nonnull (response);
         g_assert_no_error (error);
-        g_bytes_unref (response);
+        g_clear_error (&error);
         g_object_unref (msg);
         g_uri_unref (uri);
 }
@@ -1121,7 +1035,6 @@ static void
 do_one_sniffer_test (SoupSession  *session,
                      const char   *path,
                      gsize         expected_size,
-                     const char   *expected_type,
                      gboolean      should_sniff,
                      GMainContext *async_context)
 {
@@ -1155,7 +1068,7 @@ do_one_sniffer_test (SoupSession  *session,
         if (should_sniff) {
                 soup_test_assert (g_object_get_data (G_OBJECT (msg), "content-sniffed") != NULL,
                                   "content-sniffed did not get emitted");
-                g_assert_cmpstr (sniffed_type, ==, expected_type);
+                g_assert_cmpstr (sniffed_type, ==, "text/plain");
         } else {
                 soup_test_assert (g_object_get_data (G_OBJECT (msg), "content-sniffed") == NULL,
                                   "content-sniffed got emitted without a sniffer");
@@ -1181,11 +1094,9 @@ do_sniffer_async_test (Test *test, gconstpointer data)
         if (should_content_sniff)
                 soup_session_add_feature_by_type (test->session, SOUP_TYPE_CONTENT_SNIFFER);
 
-        do_one_sniffer_test (test->session, "/", 11, "text/plain", should_content_sniff, async_context);
-        do_one_sniffer_test (test->session, "/large", (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1, "text/plain", should_content_sniff, async_context);
-        do_one_sniffer_test (test->session, "/no-content", 0, "text/plain", should_content_sniff, async_context);
-        do_one_sniffer_test (test->session, "/no-content-but-has-content-type", 0, "text/javascript", should_content_sniff, async_context);
-        do_one_sniffer_test (test->session, "/empty-but-has-content-type", 0, "text/javascript", should_content_sniff, async_context);
+        do_one_sniffer_test (test->session, "/", 11, should_content_sniff, async_context);
+        do_one_sniffer_test (test->session, "/large", (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1, should_content_sniff, async_context);
+        do_one_sniffer_test (test->session, "/no-content", 0, should_content_sniff, async_context);
 
         g_main_context_unref (async_context);
 }
@@ -1198,12 +1109,9 @@ do_sniffer_sync_test (Test *test, gconstpointer data)
         if (should_content_sniff)
                 soup_session_add_feature_by_type (test->session, SOUP_TYPE_CONTENT_SNIFFER);
 
-        do_one_sniffer_test (test->session, "/", 11, "text/plain", should_content_sniff, NULL);
-        do_one_sniffer_test (test->session, "/large", (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1, "text/plain", should_content_sniff, NULL);
-        do_one_sniffer_test (test->session, "/no-content", 0, "text/plain", should_content_sniff, NULL);
-        do_one_sniffer_test (test->session, "/no-content-but-has-content-type", 0, "text/javascript", should_content_sniff, NULL);
-        do_one_sniffer_test (test->session, "/empty-but-has-content-type", 0, "text/javascript", should_content_sniff, NULL);
-
+        do_one_sniffer_test (test->session, "/", 11, should_content_sniff, NULL);
+        do_one_sniffer_test (test->session, "/large", (LARGE_N_CHARS * LARGE_CHARS_REPEAT) + 1, should_content_sniff, NULL);
+        do_one_sniffer_test (test->session, "/no-content", 0, should_content_sniff, NULL);
 }
 
 static void
@@ -1221,7 +1129,6 @@ do_timeout_test (Test *test, gconstpointer data)
         response = soup_test_session_async_send (test->session, msg, NULL, &error);
         g_assert_null (response);
         g_assert_error (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT);
-        g_clear_error (&error);
         g_object_unref (msg);
         g_uri_unref (uri);
 }
@@ -1244,30 +1151,6 @@ do_connection_closed_test (Test *test, gconstpointer data)
         g_uri_unref (uri);
 }
 
-static void
-do_broken_pseudo_header_test (Test *test, gconstpointer data)
-{
-	char *path;
-	SoupMessage *msg;
-	GUri *uri;
-	GBytes *body = NULL;
-	GError *error = NULL;
-
-	uri = g_uri_parse_relative (base_uri, "/ag", SOUP_HTTP_URI_FLAGS, NULL);
-
-	/* an ugly cheat to construct a broken URI, which can be sent from other libs */
-	path = (char *) g_uri_get_path (uri);
-	path[1] = '%';
-
-	msg = soup_message_new_from_uri (SOUP_METHOD_GET, uri);
-	body = soup_test_session_async_send (test->session, msg, NULL, &error);
-	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_PARTIAL_INPUT);
-	g_assert_null (body);
-	g_clear_error (&error);
-	g_object_unref (msg);
-	g_uri_unref (uri);
-}
-
 static gboolean
 unpause_message (SoupServerMessage *msg)
 {
@@ -1304,14 +1187,6 @@ server_handler (SoupServer        *server,
                 }
         } else if (strcmp (path, "/no-content") == 0) {
                 soup_server_message_set_status (msg, SOUP_STATUS_NO_CONTENT, NULL);
-        } else if (strcmp (path, "/no-content-but-has-content-type") == 0) {
-                soup_message_headers_set_content_type (soup_server_message_get_response_headers (msg), "text/javascript", NULL);
-                soup_server_message_set_status (msg, SOUP_STATUS_NO_CONTENT, NULL);
-        } else if (strcmp (path, "/empty-but-has-content-type") == 0) {
-                soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
-                soup_server_message_set_response (msg, "text/javascript",
-                                                  SOUP_MEMORY_STATIC,
-                                                  NULL, 0);
         } else if (strcmp (path, "/large") == 0) {
                 int i, j;
                 SoupMessageBody *response_body;
@@ -1321,14 +1196,10 @@ server_handler (SoupServer        *server,
                 response_body = soup_server_message_get_response_body (msg);
                 for (i = 0; i < LARGE_N_CHARS; i++, letter++) {
                         GString *chunk = g_string_new (NULL);
-                        GBytes *bytes;
 
                         for (j = 0; j < LARGE_CHARS_REPEAT; j++)
                                 chunk = g_string_append_c (chunk, letter);
-
-                        bytes = g_string_free_to_bytes (chunk);
-                        soup_message_body_append_bytes (response_body, bytes);
-                        g_bytes_unref (bytes);
+                        soup_message_body_append_bytes (response_body, g_string_free_to_bytes (chunk));
                 }
                 soup_message_body_append (response_body, SOUP_MEMORY_STATIC, "\0", 1);
 
@@ -1372,8 +1243,8 @@ server_handler (SoupServer        *server,
                 SoupMessageHeaders *response_headers;
 
                 response_headers = soup_server_message_get_response_headers (msg);
-                /* Use soup_message_headers_append_common with trusted_value=TRUE to skip the validation check. */
-                soup_message_headers_append_common (response_headers, SOUP_HEADER_CONTENT_TYPE, "\r", TRUE);
+                /* Use soup_message_headers_append_common to skip the validation check. */
+                soup_message_headers_append_common (response_headers, SOUP_HEADER_CONTENT_TYPE, "\r");
                 soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
         } else if (strcmp (path, "/invalid-header-rfc9113") == 0) {
                 SoupMessageHeaders *response_headers;
@@ -1490,18 +1361,6 @@ main (int argc, char **argv)
                     setup_session,
                     do_paused_async_test,
                     teardown_session);
-        g_test_add ("/http2/flow-control/large/async", Test, GINT_TO_POINTER (TRUE),
-                    setup_session,
-                    do_flow_control_large_test,
-                    teardown_session);
-        g_test_add ("/http2/flow-control/large/sync", Test, GINT_TO_POINTER (FALSE),
-                    setup_session,
-                    do_flow_control_large_test,
-                    teardown_session);
-        g_test_add ("/http2/flow-control/multiplex/async", Test, NULL,
-                    setup_session,
-                    do_flow_control_multi_message_async_test,
-                    teardown_session);
         g_test_add ("/http2/connections", Test, NULL,
                     setup_session,
                     do_connections_test,
@@ -1580,10 +1439,6 @@ main (int argc, char **argv)
                     setup_session,
                     do_connection_closed_test,
                     teardown_session);
-        g_test_add ("/http2/broken-pseudo-header", Test, NULL,
-                    setup_session,
-                    do_broken_pseudo_header_test,
-                    teardown_session);
 
 	ret = g_test_run ();
 
